/*
 * bdd/PompierMysql.java
 */
package bdd;

import beans.Caserne;
import beans.Pompier;
import beans.Statut;
import beans.TypePers;
import beans.Grade;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

public class PompierMySQL {

    private final Connection theConnection = Connexion.getConnect("172.20.10.5",  
                                                                "sdis29", 
                                                                "adminBDsdis", 
                                                                "mdpBDsdis"); 
    // Tableau des attributs mis à jour
    // 1ère dimension : noms des attributs en bdd
    // 2ème dimension : noms des champs transmis
    // 3ème dimension : type de données, i pour Integet, s pour String
    private String[][] lesAttributs = {
        {"id", "idCaserne", "nom", "prenom", "statut", "typePers", "mail", "login", "mdp", "adresse", "cp", "ville", "bip", "grade", "commentaire"},
        {"idPompier", "idCaserne", "nom", "prenom", "statut", "type", "mail", "login", "mdp", "adresse", "cp", "ville", "bip", "grade", "obs"},
        {"i", "i", "s", "s", "i", "i", "s", "s", "s", "s", "s", "s", "s", "i", "s"}
    };

    /**
     * Recherche du pompier ayant le login et le mdp passé en paramètres
     *
     * @param login
     * @param mdp
     * @return Pompier correspondant ou null si non trouvé
     */
    public Pompier readAuthentif(String login, String mdp) {
        Pompier lePompier = null;
        try {
            PreparedStatement prepStmt;
            String sql = "SELECT * FROM pompier WHERE login=? AND mdp=? ";

            prepStmt = theConnection.prepareStatement(sql);
            prepStmt.setString(1, login);
            prepStmt.setString(2, mdp);
            ResultSet result = prepStmt.executeQuery();
            if (result.next()) {
                CaserneMySQL laCaserneMySQL = new CaserneMySQL();
                Caserne laCaserne = laCaserneMySQL.read(result.getInt(2));
                lePompier = constituerLePompier(result, laCaserne);
            }
            prepStmt.close();
        } catch (SQLException ex) {
            System.out.println("SQLExeption : " + ex.getMessage());
            System.out.println("SQLState : " + ex.getSQLState());
            System.out.println("Code erreur : " + ex.getErrorCode());
        }
        return lePompier;
    }

    /**
     * Recherche les pompiers volontaires de la caserne
     *
     * @param idCaserne
     * @return collection des pompiers concernés
     */
    public ArrayList<Pompier> readLesPompiersCaserne(int idCaserne) {
        ArrayList<Pompier> lesPompiers = new ArrayList();
        try {
            PreparedStatement prepStmt;
            String sql = "SELECT * FROM pompier WHERE idCaserne=? AND statut=1 AND typePers=2 ORDER BY nom, prenom";

            prepStmt = theConnection.prepareStatement(sql);
            prepStmt.setInt(1, idCaserne);
            ResultSet result = prepStmt.executeQuery();
            CaserneMySQL laCaserneMySQL = new CaserneMySQL();
            Caserne laCaserne = laCaserneMySQL.read(idCaserne);
            while (result.next()) {
                Pompier unPompier = constituerLePompier(result, laCaserne);
                lesPompiers.add(unPompier);
            }
            prepStmt.close();
        } catch (SQLException ex) {
            System.out.println("SQLExeption : " + ex.getMessage());
            System.out.println("SQLState : " + ex.getSQLState());
            System.out.println("Code erreur : " + ex.getErrorCode());
        }
        return lesPompiers;
    }

    public Pompier constituerLePompier(ResultSet result, Caserne laCaserne) {
        Pompier lePompier = null;
        StatutMySQL leStatutMySQL = new StatutMySQL();
        GradeMySQL leGradeMySQL = new GradeMySQL();
        TypePersoMySQL leTypePersoMySQL = new TypePersoMySQL();
        try {
          
        Statut leStatut = leStatutMySQL.read(result.getInt("statut"));
        TypePers leTypePers = leTypePersoMySQL.read(result.getInt("typePers"));
        Grade leGrade = leGradeMySQL.read(result.getInt("grade"));
        
            lePompier = new Pompier(
                    result.getInt(1),
                    laCaserne.getId(),
                    result.getString(3),
                    result.getString(4),
                    leStatut.getIdStatut(),
                    leTypePers.getIdTypePers(),
                    result.getString(7),
                    result.getString(8),
                    result.getString(9),
                    result.getString(10),
                    result.getString(11),
                    result.getString(12),
                    result.getString(13),
                    leGrade.getIdGrade(),
                    result.getString(16));
            //System.out.println("lePompier " + lePompier);
        } catch (SQLException ex) {
            Logger.getLogger(PompierMySQL.class.getName()).log(Level.SEVERE, null, ex);
        }
        return lePompier;
    }

    public Pompier create(HashMap<String, Object> lesParametres) {
        Pompier lePompierCree = null;
        
        String valeurs [] = new String[lesAttributs[1].length];
        String sql = "INSERT INTO pompier (";
        int nbAttributsMaj = 0;
        String virgule = "";
        for (HashMap.Entry unParam : lesParametres.entrySet()) {
     
            int i = 0;
            while (i < lesAttributs[1].length && !(lesAttributs[1][i].equals(unParam.getKey()))) {                
                i++;
            }
            if (i < lesAttributs[0].length) {
                sql += virgule + lesAttributs[0][i];
                valeurs[nbAttributsMaj] = "("+lesAttributs[2][i]+")" + unParam.getValue();
                nbAttributsMaj++;
                virgule = ", ";
            }
        }
        sql += ", dateEnreg) values (?";
        for (int i=0; i<nbAttributsMaj; i++ ) {
            sql+= ", ?";
        }
        sql += ");";
        
        // Alimentation des valeurs de la requête
        PreparedStatement prepStmt;
        try {
            prepStmt = theConnection.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
            for (int i=0; i<nbAttributsMaj; i++) {
                if (valeurs[i].substring(0, 3).equals("(s)")) {
                    prepStmt.setString(i+1, valeurs[i].substring(3));
                } else {
                    prepStmt.setInt(i+1, Integer.parseInt(valeurs[i].substring(3)));
                }               
            }
            prepStmt.setTimestamp(nbAttributsMaj+1, new Timestamp(Calendar.getInstance().getTimeInMillis()));
            System.out.println("sql "+prepStmt);
            
            int nbLigne = prepStmt.executeUpdate();

            //Recherche du no du pompier créé
            if (nbLigne > 0) {
                ResultSet result = prepStmt.getGeneratedKeys();
                if (result.first()) {
                    int id = result.getInt(1);
                    lePompierCree = read(id);
                    System.out.println("lePompierCree : "+ lePompierCree);                    
                }
            }
            prepStmt.close();
        } catch (SQLException ex) {
            System.out.println("SQLExeption : " + ex.getMessage());
            System.out.println("SQLState : " + ex.getSQLState());
            System.out.println("Code erreur : " + ex.getErrorCode());
        }
         
        return lePompierCree;
    }

    public Pompier read(int idP) {
        Pompier unPompier = null;
        try {
            PreparedStatement prepStmt;
            String sql = "SELECT * FROM pompier WHERE idPompier=? ";

            prepStmt = theConnection.prepareStatement(sql);
            prepStmt.setInt(1, idP);        
            ResultSet result = prepStmt.executeQuery();
            if (result.next()) {
                CaserneMySQL laCaserneMySQL = new CaserneMySQL();
                Caserne laCaserne = laCaserneMySQL.read(result.getInt(2));
                unPompier = constituerLePompier(result, laCaserne);
            }
            prepStmt.close();
        } catch (SQLException ex) {
            System.out.println("SQLExeption : " + ex.getMessage());
            System.out.println("SQLState : " + ex.getSQLState());
            System.out.println("Code erreur : " + ex.getErrorCode());
        }
        return unPompier;
    }
}
